/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.testsuites;



import java.io.IOException;
import java.util.List;



import org.thermostat.qa.annotations.TestType;
import org.thermostat.qa.annotations.TestTypes;
import org.thermostat.qa.framework.Assert;
import org.thermostat.qa.framework.ThermostatTest;


/**
 * Class StorageSmokeTest
 * contains basic smoke tests for testing thermostat storage. 
 * The storage is started and stopped. 
 * Contents of the output in both cases and presence / absence 
 * of the mongo processes are checked. 
 *
 */
@TestType(TestTypes.STORAGE_TEST)
public class StorageSmokeTest extends ThermostatTest
{

    private static final String STORAGE_MESSAGE_SERVER_SHUTDOWN_COMPLETE = "server shutdown complete: ";
    private static final String STORAGE_MESSAGE_PID = "pid: ";
    private static final String STORAGE_MESSAGE_LOG_FILE = "log file is here:";

    @Override
    protected void setUp()
    {
        // TODO Auto-generated method stub
    }

    @Override
    protected void tearDown()
    {
        // TODO Auto-generated method stub
    }

    private List<String> startStorageAndGetOutput() throws IOException
    {
        Process process = runThermostatInNewProcess("storage", "--start");
        List<String> processOutput = readProcessOutput(process);
        sleep(ONE_SEC);
        int exitValue = process.exitValue();
        Assert.assertEquals(exitValue, EXPECTED_EXIT_VALUE, "Bad thermostat exit value " + exitValue);
        return processOutput;
    }

    private List<String> stopStorageAndGetOutput() throws IOException
    {
        Process process = runThermostatInNewProcess("storage", "--stop");
        List<String> processOutput = readProcessOutput(process);
        sleep(ONE_SEC);
        int exitValue = process.exitValue();
        Assert.assertEquals(exitValue, EXPECTED_EXIT_VALUE, "Bad thermostat exit value " + exitValue);
        return processOutput;
    }

    /**
     * Method testCheckIfMongoDBIsNotRunningBeforeTest
     * tries to list all mongo processes without starting thermostat storage.
     * The output of the script should not be null, but it should empty.
     * This method uses script 
     * <a href="scripts/ps_mongo_daemon.sh">ps_mongo_daemon.sh</a>.
     * 
     * @throws IOException
     */
    public void testCheckIfMongoDBIsNotRunningBeforeTest() throws IOException
    {
        List<String> out = runHelperBashScript("ps_mongo_daemon.sh");
        Assert.assertNotNull(out, "script 'ps_mongo_daemon.sh' can't be started");
        Assert.assertEquals(out.size(), 0, "at least mongod instance are running!");
    }

    /**
     * Method testStartStorage starts thermostat storage and
     * tests that the output is not null. Then the storage is stopped again.
     * 
     * @throws IOException
     */
    public void testStartStorage() throws IOException
    {
        List<String> processOutput = startStorageAndGetOutput();
        sleep(ONE_SEC);
        stopStorage();
        Assert.assertNotNull(processOutput, "error getting standard output");
    }

    /**
     * Method testStopStorage starts thermostat storage, then it stops 
     * the storage and tests if the output is not null. 
     * 
     * @throws IOException
     */
    public void testStopStorage() throws IOException
    {
        startStorage();
        sleep(ONE_SEC);
        List<String> processOutput = stopStorageAndGetOutput();
        sleep(ONE_SEC);
        Assert.assertNotNull(processOutput, "error getting standard output");
    }

    /**
     * Method testCheckIfMongoDBIsStarted
     * lists all mongo processes and tests if this list is not null and there
     * is exactly one mongo process. 
     * This method uses script 
     * <a href="scripts/ps_mongo_daemon.sh">ps_mongo_daemon.sh</a>.
     * 
     * @throws IOException
     */
    public void testCheckIfMongoDBIsStarted() throws IOException
    {
        startStorage();
        List<String> out = runHelperBashScript("ps_mongo_daemon.sh");
        sleep(ONE_SEC);
        stopStorage();
        Assert.assertNotNull(out, "test script 'scripts/ps_mongo_daemon.sh' can't be started");
        Assert.assertEquals(out.size(), 1, "mongod was not started");
    }

    /**
     * Method testStartStorageMessage
     * starts the thermostat storage and checks the contents of the output.
     * The output should not be null, should have at least 3 lines,
     * the content of first and second line has to be as given, 
     * the prefixes of third and fourth line have to be as given.   
     * 
     * @throws IOException
     */
    public void testStartStorageMessage() throws IOException
    {
        List<String> processOutput = startStorageAndGetOutput();
        sleep(ONE_SEC);
        stopStorage();
        for (String message : processOutput)
        {
            logInfo("testStartStorageMessage", message);
        }
        Assert.assertNotNull(processOutput, "error getting standard output");
        Assert.assertTrue(processOutput.size()>=3, "error getting standard output");
        compareTwoLines(processOutput.get(0), "starting storage server...");
        compareTwoLines(processOutput.get(1), "server listening on ip: mongodb://127.0.0.1:27518");
        compareTwoLines(processOutput.get(2).substring(0, STORAGE_MESSAGE_LOG_FILE.length()), STORAGE_MESSAGE_LOG_FILE);
        compareTwoLines(processOutput.get(3).substring(0, STORAGE_MESSAGE_PID.length()), STORAGE_MESSAGE_PID);
    }

    /**
     * Method testStopStorageMessage
     * starts the thermostat storage and checks the contents of the output.
     * The output should not be null, the prefixes of first and second line
     * have to be as given.
     * 
     * @throws IOException
     */
    public void testStopStorageMessage() throws IOException
    {
        startStorage();
        sleep(ONE_SEC);
        List<String> processOutput = stopStorageAndGetOutput();
        sleep(ONE_SEC);
        for (String message : processOutput)
        {
            logInfo("testStopStorageMessage", message);
        }
        Assert.assertNotNull(processOutput, "error getting standard output");
        compareTwoLines(processOutput.get(0).substring(0, STORAGE_MESSAGE_SERVER_SHUTDOWN_COMPLETE.length()), STORAGE_MESSAGE_SERVER_SHUTDOWN_COMPLETE);
        compareTwoLines(processOutput.get(1).substring(0, STORAGE_MESSAGE_LOG_FILE.length()), STORAGE_MESSAGE_LOG_FILE);
    }

    public static void main(String[] args)
    {
        new StorageSmokeTest().doTests(args);
    }
}
